with Text_Io;

with Unix.System_Calls;
with Interfaces.C.Strings;
with Unix.Errno;
package body Unix is
   use Interfaces.C;
   use Interfaces.C.Strings;

   procedure Perror (Arg: String := "") is
      C_Ptr: Chars_Ptr := New_String(Arg);
   begin
      if Arg = "" then
         Unix.System_Calls.Perror;
      else
         Unix.System_Calls.Perror( C_Ptr );
      end if;
      Free(C_Ptr);
   end Perror;

   -- Files system calls
   function Creat(Path : String;
                  Perms: Integer) return Natural is
      C_Path: Chars_Ptr := New_String(Path);
      Fd: Int;
   begin
      Fd := Unix.System_Calls.Creat( C_Path, Int(Perms) );
      Free(C_Path);
      Unix.Errno.Check(Fd);
      return Integer(Fd);
   end Creat;

   function Open(Path : String;
                 Flags: Integer) return Natural is
      C_Path: Chars_Ptr := New_String(Path);
      Fd: Int;
   begin
      Fd := Unix.System_Calls.Open( C_Path, Int(Flags) );
      Free(C_Path);
      Unix.Errno.Check(Fd);
      return Integer(Fd);
   end Open;

   procedure Write(Fid   : Natural;
                   Buff  : String;
                   Nbytes: Natural) is
      Err   : Int;
      C_Buff: Chars_Ptr := New_String(Buff);
   begin
      Err := Unix.System_Calls.Write( Int(Fid),
                                      C_Buff,
                                      Int(Nbytes) ); -- Revisar
      Free(C_Buff);
      Unix.Errno.Check(Err);
   end Write;


--    procedure Read(Fid   : in     Integer;        -- REVISAR (FALLA)
--                Buff  :    out String;
--                Nbytes: in out Natural) is
--       C_Buff: Chars_Ptr := New_String( Buff(1..NBytes-1) );
--       Err   : Int;
--    begin
--       Err := Unix.System_Calls.Read( Int(Fid),
--                                   C_Buff,
--                                   Int(Nbytes) ); -- Revisar
--       Buff := Value(C_Buff);
--       Free(C_Buff);
--       Unix.Errno.Check(Err);
--       Nbytes := Natural(Err);
--    end Read;

   procedure Read(Fid        : in  Natural;         -- REVISAR
                  Buff       : out String;
                  Nbytes     : in  Natural;
                  Nbytes_Read: out natural) is

      function System_Read(Fid: Integer;
                           Buff: String;
                           Nbytes: Natural) return Int;
      pragma Import (C, System_Read, "read", "read");
      Err   : Int;
   begin
      Buff(1):=' ';                           -- To eliminate a warning
      Err := System_Read( Fid, Buff, NBytes);
      Unix.Errno.Check(Err);
      NBytes_Read := Natural(Err);
   end Read;




   procedure Close(Fid: Natural) is
      Err: Int;
   begin
      Err := Unix.System_Calls.Close( Int(Fid) );
      Unix.Errno.Check(Err);
   end Close;

   -- -----------------------------------------------------------------------

   function Lseek(Fid: Natural;
                  Offset: Integer;
                  Whence: Integer) return Integer is
      Err: Int;
   begin
      Err := Unix.System_Calls.Lseek( Int(Fid), long(Offset), Int(Whence) );
      Unix.Errno.Check(Err);
      return Integer(Err);
   end Lseek;

   function Dup(Fid: Natural) return Natural is
      New_Fid: Int;
   begin
      New_Fid := Unix.System_Calls.Dup( Int(Fid) );
      Unix.Errno.Check(New_Fid);
      return Integer(New_Fid);
   end Dup;

   -- Processes system calls
   function Fork return Natural is
      Pid: Int;
   begin
      Pid:= Unix.System_Calls.Fork;
      Unix.Errno.Check(Pid);
      return Natural(Pid);
   end Fork;

   procedure Execlp(Path: String;
                   Arg0: String;
                   Arg1: String := "";
                   Arg2: String := "";
                   Arg3: String := "";
                   Arg4: String := "";
                   Arg5: String := "";
                   Arg6: String := "";
                   Arg7: String := "";
                   Arg8: String := "";
                   Arg9: String := "") is
      Err: Int;
      C_Path: Chars_Ptr := New_String( Path );
      C_Arg0: Chars_Ptr := New_String( Arg0 );
      C_Arg1: Chars_Ptr := New_String( Arg1 );
      C_Arg2: Chars_Ptr := New_String( Arg2 );
      C_Arg3: Chars_Ptr := New_String( Arg3 );
      C_Arg4: Chars_Ptr := New_String( Arg4 );
      C_Arg5: Chars_Ptr := New_String( Arg5 );
      C_Arg6: Chars_Ptr := New_String( Arg6 );
      C_Arg7: Chars_Ptr := New_String( Arg7 );
      C_Arg8: Chars_Ptr := New_String( Arg8 );
      C_Arg9: Chars_Ptr := New_String( Arg9 );
   begin
      if Arg1 = "" then
         Err := Unix.System_Calls.Execlp( C_Path, C_Arg0 );
      elsif Arg2 = "" then
         Err := Unix.System_Calls.Execlp( C_Path, C_Arg0, C_Arg1 );
      elsif Arg3 = "" then
         Err := Unix.System_Calls.Execlp( C_Path, C_Arg0, C_Arg1,
                                          C_Arg2 );
      elsif Arg4 = "" then
         Err := Unix.System_Calls.Execlp( C_Path, C_Arg0, C_Arg1,
                                          C_Arg2, C_Arg3 );
      elsif Arg5 = "" then
         Err := Unix.System_Calls.Execlp( C_Path, C_Arg0, C_Arg1,
                                          C_Arg2, C_Arg3, C_Arg4 );
      elsif Arg6 = "" then
         Err := Unix.System_Calls.Execlp( C_Path, C_Arg0, C_Arg1,
                                          C_Arg2, C_Arg3, C_Arg4,
                                          C_Arg5 );
      elsif Arg7 = "" then
         Err := Unix.System_Calls.Execlp( C_Path, C_Arg0, C_Arg1,
                                          C_Arg2, C_Arg3, C_Arg4,
                                          C_Arg5, C_Arg6 );
      elsif Arg8 = "" then
         Err := Unix.System_Calls.Execlp( C_Path, C_Arg0, C_Arg1,
                                          C_Arg2, C_Arg3, C_Arg4,
                                          C_Arg5, C_Arg6, C_Arg7 );
      elsif Arg9 = "" then
         Err := Unix.System_Calls.Execlp( C_Path, C_Arg0, C_Arg1,
                                          C_Arg2, C_Arg3, C_Arg4,
                                          C_Arg5, C_Arg6, C_Arg8 );
      else
         Err := Unix.System_Calls.Execlp( C_Path, C_Arg0, C_Arg1,
                                          C_Arg2, C_Arg3, C_Arg4,
                                          C_Arg5, C_Arg6, C_Arg8,
                                          C_Arg9 );
      end if;
      Free(C_Path);
      Free(C_Arg0);
      Free(C_Arg1);
      Free(C_Arg2);
      Free(C_Arg3);
      Free(C_Arg4);
      Free(C_Arg5);
      Free(C_Arg6);
      Free(C_Arg7);
      Free(C_Arg8);
      Free(C_Arg9);
      Unix.Errno.Check(Err);
   end Execlp;


--   type T_Argv is array(Positive range <>) of T_Str_Ptr;
--   function Execvp(File: String;
--                 Argv: T_Argv) return Integer;

   function Wait(Status: Natural_Ptr) return Natural is
      Err: Int;
   begin
      Err := Unix.System_Calls.Wait( Status );
      Unix.Errno.Check(Err);
      return Natural(Err);
   end Wait;


   procedure Sys_Exit(Status: Integer) is
   begin
      Unix.System_Calls.Sys_Exit( Int(Status) );
   end Sys_Exit;

   -- Processes communication
   procedure Pipe(Fildes: out T_Fildes) is
      Err: Int;
   begin
      Fildes(0):=0;
      Err := Unix.System_Calls.Pipe( Fildes );
      Unix.Errno.Check(Err);
   end Pipe;


end Unix;
